import {
    HttpException,
    HttpStatus,
    Injectable,
    InternalServerErrorException,
    UnauthorizedException
} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {ResponseUtils} from "src/utils/response.utils";
import JwtHelper from "../../core/jwt/jwt.helper";
import {Constants} from "../../utils/constants";
import CryptoUtils from "../../utils/crypto.utils";
import {CreateUserDto} from "../users/dto/create-user.dto";
import {User} from "../users/entities/user.entity";
import {UsersRepository} from "../users/user.repository";
import {LoginAuthDto} from "./dto/login-auth.dto";
import {RegisterAuthDto} from "./dto/register-auth.dto";
import {ResetPasswordRequestDto} from "./dto/reset-password.dto";
import {v4 as uuidv4} from "uuid";
import {ChangePasswordTokenRepository} from "../change-password-token/change-password-token.repository";
import {CreateChangePasswordTokenDto} from "../change-password-token/dto/create-change-password-token.dto";
import {MailSenderService} from "../mail-sender/mail-sender.service";
import {ResetPasswordMailDto} from "../mail-sender/dto/create-reset-password-mail.dto";
import {CreateMailSenderDto} from "../mail-sender/dto/create-mail-sender.dto";
import {AdminNotifyDto} from "../mail-sender/dto/admin-notify.dto";
import {EmailNotifierRepository} from "../email-notifier/email-notifier-repository";
import {GoogleLoginDto} from "./dto/create-google-login.dto";
import {AlpineiqService} from "../alpineiq/alpineiq.service";
import {CheckVerifyHashPasswordDto} from "./dto/check-verify-hash-password-token";
import {UpdatePasswordUsingVerifyHashPasswordTokenDto} from "./dto/update-password-using-verify-hash-password-token";

@Injectable()
export class AuthService {
    constructor(
        @InjectRepository(UsersRepository)
        private readonly repository: UsersRepository,
        @InjectRepository(ChangePasswordTokenRepository)
        private readonly changePasswordTokenRepo: ChangePasswordTokenRepository,
        private readonly jwtHelper: JwtHelper,
        private readonly servMail: MailSenderService,
        private readonly emailNotifierRepository: EmailNotifierRepository,
        private readonly alpineiqService: AlpineiqService
    ) {
    }

    async login(dto: LoginAuthDto): Promise<any> {
        const {email, password} = dto;
        const data =
            (await this.repository.findByEmailAddressAndActive(email, 3)) ||
            (await this.repository.findByEmailAddressAndActive(email, 2));

        if (data instanceof Error) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }
        if (!data) {
            throw new HttpException("User not created", HttpStatus.NOT_FOUND);
        }

        const resData = await this.authenticateUser(password, data);
        if (resData == null) {
            // handle unauth
            throw new UnauthorizedException(Constants.UNAUTH_REQ);
        }
        return resData;
    }

    async loginAdmin(dto: LoginAuthDto): Promise<any> {
        const {email, password} = dto;

        const data = await this.repository.findByEmailAddressAndActive(email, 2);

        if (data instanceof Error) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }
        if (!data) {
            throw new HttpException("User not created", HttpStatus.NOT_FOUND);
        }

        const resData = await this.authenticateUser(password, data);
        if (resData == null) {
            // handle unauth
            throw new UnauthorizedException(Constants.UNAUTH_REQ);
        }
        return resData;
    }

    async authenticateUser(password: string, user: User): Promise<any> {
        const {password: passwordHash, ...entityWithoutPassword} = user;
        const matched = await CryptoUtils.compare(password, passwordHash);
        if (matched) {
            const token = this.jwtHelper.generateToken(
                {
                    userId: entityWithoutPassword.id
                },
                "7d"
            );
            return {
                token: token,
                user: entityWithoutPassword
            };
        }
        return null;
    }

    async register(dto: RegisterAuthDto): Promise<User | Error> {
        const {firstName, lastName, birthdayDate, email, password, phone} = dto;

        const isEmailExists = await this.repository.findByEmailAddress(email);
        if (isEmailExists) {
            throw new HttpException("Email Already Exists", HttpStatus.BAD_REQUEST);
        }
        const createUserDto = {
            firstName,
            lastName,
            birthdayDate,
            email,
            password,
            phone,
            role: 3
        } as CreateUserDto;
        const data = await this.repository.createEntity(createUserDto);

        if (data instanceof Error) {
            return new InternalServerErrorException(data);
        } else {
            const loginData = await this.login({
                email: email,
                password: password
            });

            // create new loyalty contact 
            const createNewContact = await this.alpineiqService.personaCreateNewContact({
                firstName: dto.firstName,
                lastName: dto.lastName,
                mobilePhone: dto.phone,
                email: dto.email,
            });

            //user getting email
            let mailDto = {
                to: [email],
                from: process.env.EMAIL_SENDER_MAIL,
                subject: "Dispo Shop User Account Creation Notification",
                username: firstName + " " + lastName,
                password: password
            };

            const userMail = await this.servMail.accocuntCreateCustomerNotify(
                mailDto as CreateMailSenderDto
            );
            console.log(userMail);
            const admins: any = await this.emailNotifierRepository.findOnlyEmail();

            if (admins && admins.length > 0) {
                for (const element of admins) {
                    let mailAdminDto = {
                        to: [element.email],
                        from: process.env.EMAIL_SENDER_MAIL,
                        subject: "New Customer Notification.",
                        name: firstName + " " + lastName,
                        email: email
                    };
                    const adminMail = await this.servMail.adminNotify(
                        mailAdminDto as AdminNotifyDto
                    );
                    console.log(adminMail);
                }
            }
            return ResponseUtils.successResponseHandler(
                201,
                "Customer registered successfully!",
                "data",
                loginData
            );
        }
    }

    async changePassword(
        currentPassword: string,
        newPassword: string,
        request: any
    ): Promise<User | Error> {
        const userId = request.user.id;
        const user = await this.repository.findOne(userId);
        const compare = await CryptoUtils.compare(currentPassword, user.password);

        if (!compare) {
            throw new HttpException("Password Does not Match", HttpStatus.NOT_FOUND);
        }
        await this.repository.update(userId, {
            password: await CryptoUtils.encrypt(newPassword)
        });

        return ResponseUtils.successResponseHandler(
            200,
            "Password change successfully!",
            "data",
            ""
        );
    }

    async forgotPassword(dto: ResetPasswordRequestDto): Promise<User | Error> {
        let userCheck = await this.repository.findByEmailAddress(dto.email);
        if (!userCheck) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }

        const tokenId = uuidv4();
        let cpr = {
            email: dto.email,
            token: tokenId
        };
        let data = await this.changePasswordTokenRepo.createChangePasswordToken(
            cpr as CreateChangePasswordTokenDto
        );

        console.log("createChangePasswordToken=", data);
        let mailDto = {
            to: [dto.email],
            from: process.env.EMAIL_SENDER_MAIL,
            subject: "Dispo Shop password reset request",
            username: userCheck["fullName"],
            token: cpr.token
        };

        this.servMail.sendPasswordResetRequestMail(mailDto as ResetPasswordMailDto);

        return ResponseUtils.buildData(true);
    }

    async validateRequest(req: any): Promise<boolean> {
        try {
            const bearerToken = req.headers.authorization;
            const bearer = "Bearer ";
            if (!bearerToken || !bearerToken.startsWith(bearer)) {
                return false;
            }
            const token = bearerToken.replace(bearer, "");
            const jwtPayload = this.jwtHelper.verifyToken(token);
            const data = await this.repository.findOne({
                where: {
                    id: jwtPayload.userId,
                    isDeleted: false
                }
            });
            if (!data || data instanceof Error) {
                return false;
            }

            req.data = data;

            return true;
        } catch (e) {
            return false;
        }
    }

    async doGoogleLoginService(dto: GoogleLoginDto) {
        let loginResponse = null;
        let checkUserEmail = await this.repository.findByEmailAddress(dto.email);
        if (!checkUserEmail) {
            console.log('New User Created');
            let userCreateDto = {
                firstName: dto.name,
                lastName: dto.name,
                birthdayDate: '',
                email: dto.email,
                password: dto.token,
                phone: '',
                role: 3
            };

            const data = await this.repository.createEntity(
                userCreateDto as CreateUserDto,
            );
            if (!data) {
                console.log('Data User Not Created');
            } else {
                console.log('Data =', data);
            }

            let dtoLogin = {
                email: dto.email,
                password: dto.token,
            };
            loginResponse = await this.login(dtoLogin as LoginAuthDto);
        } else {
            console.log('User Already exists');
            await this.repository.updateUserPassword(
                checkUserEmail['id'],
                await CryptoUtils.encrypt(dto.token),
            );
            let dtoLogin = {
                email: dto.email,
                password: dto.token,
            };
            loginResponse = await this.login(dtoLogin as LoginAuthDto);
        }
        return loginResponse;
    }

    async verifyHashPasswordTokenRequest(dto: CheckVerifyHashPasswordDto) {
        let userCheck = await this.changePasswordTokenRepo.findByHashToken(dto.hashPass);
        if (!userCheck) {
            return ResponseUtils.buildData(false);
        } else {
            return ResponseUtils.buildData(true);
        }
    }

    async updatePasswordUsingTokenRequest(
        dto: UpdatePasswordUsingVerifyHashPasswordTokenDto
    ) {
        let userCheck = await this.changePasswordTokenRepo.findByHashToken(dto.passToken);
        if (!userCheck) {
            return ResponseUtils.buildData(false);
        } else {
            let userInfo = await this.repository.findByEmailAddress(userCheck['email']);
            if (!userInfo) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }
            let updPass = await this.repository.updateUserPassword(
                userInfo['id'],
                await CryptoUtils.encrypt(dto.password)
            );
            return ResponseUtils.buildData(true);
        }
    }
}
